import unittest
from commons.launcher.RepeatMaskerProgramLauncher import RepeatMaskerProgramLauncher


class Test_RepeatMaskerProgramLauncher( unittest.TestCase ):
    
    def setUp( self ):
        self._i = RepeatMaskerProgramLauncher()
        
        
    def tearDown( self ):
        self._i = None
        
        
    def test_getSpecificHelpAsString( self ):
        exp = ""
        exp += "\nspecific options:"
        exp += "\n     -s: name of the subject file (repeats, format='fasta')"
        exp += "\n     -n: nb of processors to use in parallel (default='%i')" % ( self._i.getNbProcessors() )
        exp += "\n     -g: calculate the GC content"
        exp += "\n     -b: skip bacterial insertion element check"
        exp += "\n     -l: does not mask low-complexity DNA or simple repeats"
        exp += "\n     -m: only masks low complex/simple repeats (no interspersed repeats)"
        exp += "\n     -o: name of the output file"
        exp += "\n         with -s: format='align', default=inFile+'.cat.align')"
        exp += "\n         with -m: format='path', default=inFile+'.cat.path')"
        obs = self._i.getSpecificHelpAsString()
        self.assertEqual( exp, obs )
        
        
    def test_setASpecificAttributeFromCmdLine( self ):
        self._i.setASpecificAttributeFromCmdLine( "-s", "dummySubjectFile.fa" )
        self.assertEqual( "dummySubjectFile.fa", self._i.getSubjectFile() )
        
        self._i.setASpecificAttributeFromCmdLine( "-n", "2" )
        self.assertEqual( 2, self._i.getNbProcessors() )
        
        self._i.setASpecificAttributeFromCmdLine( "-g" )
        self.assertEqual( True, self._i.getCalculateGCcontent() )
        
        self._i.setASpecificAttributeFromCmdLine( "-b" )
        self.assertEqual( True, self._i.getSkipBacterialIsCheck() )
        
        self._i.setASpecificAttributeFromCmdLine( "-l" )
        self.assertEqual( False, self._i.getMaskSsr() )
        
        self._i.setASpecificAttributeFromCmdLine( "-o", "dummyOutFile.align" )
        self.assertEqual( "dummyOutFile.align", self._i.getOutputFile() )
        
        
    def test_setWrapperCommandLine( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        self._i.setSubjectFile( "dummySubjectFile.fa" )
        exp = "RepeatMaskerProgramLauncher.py"
        exp += " -i %s" % ( "dummyInFile.fa" )
        exp += " -s %s" % ( "dummySubjectFile.fa" )
        exp += " -n 1"
        self._i.setWrapperCommandLine()
        obs = self._i.getWrapperCommandLine()
        self.assertEqual( exp, obs )
        
        
    def test_setProgramCommandLine( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        self._i.setSubjectFile( "dummySubjectFile.fa" )
        self._i.setCalculateGCcontent()
        self._i.setSkipBacterialIsCheck()
        exp = "RepeatMasker -dir . -pa 1 -gccalc -no_is -nolow -lib dummySubjectFile.fa dummyInFile.fa"
        self._i.setProgramCommandLine()
        obs = self._i.getProgramCommandLine()
        self.assertEqual( exp, obs )
        
        
    def test_setProgramCommandLine_withoutSubjects_withMaskSsr( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        self._i.setCalculateGCcontent()
        self._i.setSkipBacterialIsCheck()
        exp = "RepeatMasker -dir . -pa 1 -gccalc -no_is -nolow dummyInFile.fa"
        self._i.setProgramCommandLine()
        obs = self._i.getProgramCommandLine()
        self.assertEqual( exp, obs )
        
        
    def test_setListFilesToKeep_onlyInFile( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        self._i.setOnlySsr()
        lExp = [ "dummyInFile.fa.cat", "dummyInFile.fa.cat.path" ]
        self._i.setListFilesToKeep()
        lObs = self._i.getListFilesToKeep()
        lExp.sort()
        lObs.sort()
        self.assertEqual( lExp, lObs )
        
        
    def test_setListFilesToKeep_withSubjectFile( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        self._i.setSubjectFile( "dummySbjFile.fa" )
        lExp = [ "dummyInFile.fa.cat", "dummyInFile.fa.cat.align" ]
        self._i.setListFilesToKeep()
        lObs = self._i.getListFilesToKeep()
        lExp.sort()
        lObs.sort()
        self.assertEqual( lExp, lObs )
        
        
    def test_setListFilesToKeep_inFileAndOutFile( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        self._i.setOutputFile( "dummyOutFile.align" )
        lExp = [ "dummyInFile.fa.cat", "dummyOutFile.align" ]
        self._i.setListFilesToKeep()
        lObs = self._i.getListFilesToKeep()
        lExp.sort()
        lObs.sort()
        self.assertEqual( lExp, lObs )
        
        
    def test_setListFilesToKeep_inFileAndOutFileAndSsr( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        self._i.unsetMaskSsr()
        self._i.setOutputFile( "dummyOutFile.path" )
        lExp = [ "dummyInFile.fa.cat", "dummyOutFile.path" ]
        self._i.setListFilesToKeep()
        lObs = self._i.getListFilesToKeep()
        lExp.sort()
        lObs.sort()
        self.assertEqual( lExp, lObs )
        
        
    def test_setListFilesToRemove( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        lExp = [ "dummyInFile.fa.out", \
                 "dummyInFile.fa.log", \
                 "dummyInFile.fa.stderr", \
                 "dummyInFile.fa.ori.out", \
                 "dummyInFile.fa.masked", \
                 "dummyInFile.fa.tbl", \
                 "dummyInFile.fa.ref" ]
        self._i.setListFilesToRemove()
        lObs = self._i.getListFilesToRemove()
        lExp.sort()
        lObs.sort()
        self.assertEqual( lExp, lObs )
        
        
    def test_setSummary( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        self._i.setSubjectFile( "dummySubjectFile.fa" )
        exp = "input file: %s" % ( self._i.getInputFile() )
        exp += "\nsubject file: %s" % ( self._i.getSubjectFile() )
        exp += "\nnb processors: %i" % ( self._i.getNbProcessors() )
        exp += "\nmask low-complexity DNA or simple repeats"
        exp += "\noutput file: %s" % ( "dummyInFile.fa.cat.path" )
        self._i.setSummary()
        obs = self._i.getSummary()
        self.assertEqual( exp, obs )
        
        
    def test_setSummary_noMaskSsr( self ):
        self._i.setInputFile( "dummyInFile.fa" )
        self._i.setSubjectFile( "dummySubjectFile.fa" )
        self._i.unsetMaskSsr()
        exp = "input file: %s" % ( self._i.getInputFile() )
        exp += "\nsubject file: %s" % ( self._i.getSubjectFile() )
        exp += "\nnb processors: %i" % ( self._i.getNbProcessors() )
        exp += "\noutput file: %s" % ( "dummyInFile.fa.cat.align" )
        self._i.setSummary()
        obs = self._i.getSummary()
        self.assertEqual( exp, obs )
        
if __name__ == "__main__":
        unittest.main()